/*
 * File:        remoting.js
 * CVS:         $Id$
 * Description: Cross domain async javascript remoting library using iframes
 * Author:			Allan Jardine
 * Created:		  20-01-08
 * Modified:		$Date$ by $Author$
 * Language:		JavaScript
 * Project:		  Design
 * License:     GNU GPL v2.0
 * 
 * Copyright 2007-2008 Allan Jardine. All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * version 2.0 as published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */


if ( typeof Remoting != 'undefined' )
{
	Remoting = undefined;
}

/*
 * Variable: Remoting
 * Purpose:	 Container object for Remoting
 * Scope:	   Global
 */
var Remoting = {
	/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
	 * API Vars
	 */
	sResponse: null,
	
	
	
	/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
	 * Internal Vars
	 *
	 * Variable: aQueue
	 * Purpose:	 Array of ints with the indexs of the request objects
	 * Scope:	   Remoting
	 */
	aQueue: new Array(),
	
	
	/*
	 * Variable: aoRequests
	 * Purpose:	 List of request objects
	 * Scope:	   Remoting
	 */
	aoRequests: new Array(),
	
	
	/*
	 * Variable: iRequestingIndex
	 * Purpose:	 Index of the request object we are currently 'doing'
	 * Scope:	   Remoting
	 */
	iRequestingIndex: null,
	
	
	/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
	 * Internal functions
	 * 
	 * Function: _fnReply
	 * Purpose:  Deal with a loaded iframe
	 * Returns:  -
	 * Inputs:   -
	 */
	_fnReply: function (  )
	{
		var nIFrame = document.getElementById('RemotingIFrame');
		
		/* We actaully go looking for an error in this case as browsers prevent
		 * us from reading the innerHTML of a XSS iframe
		 */
		try {
			var sTest = nIFrame.contentWindow.document.body.innerHTML;
			// Managed to read the iframe - must be in the same domain - good stuff
			if ( sTest.match( /[Done]/ ) )
			{
				/* Delete the IFrame and clean up */
				nIFrame.parentNode.removeChild( nIFrame );
				this.iRequestingIndex = null;
				this.aQueue.splice( 0, 1 );
				this._fnQueue();
			}
			else
			{
				setTimeout( function() { Remoting._fnReply(); }, 20 );
			}
			return;
		}
		catch (e)
		{
			/* Delete the IFrame and clean up */
			nIFrame.parentNode.removeChild( nIFrame );
			this.iRequestingIndex = null;
			this.aQueue.splice( 0, 1 );
			this._fnQueue();
		}
	},
	
	
	/*
	 * Function: _fnQueue
	 * Purpose:  Make a request from the queue
	 * Returns:  -
	 * Inputs:   -
	 */
	_fnQueue: function ( )
	{
		if ( typeof this.aQueue[0] == 'undefined' || this.iRequestingIndex != null )
		{
			return -1;
		}
		
		this.iRequestingIndex = this.aQueue[0];
		
		/* Create an IFrame to do the request */
		nIFrame = document.createElement('iframe');
		nIFrame.setAttribute( 'id', 'RemotingIFrame' );
    nIFrame.style.border='0px';
    nIFrame.style.width='0px';
    nIFrame.style.height='0px';
		
    document.body.appendChild( nIFrame );
		var nContentWindow = nIFrame.contentWindow;
		nContentWindow.document.open();
		nContentWindow.document.close();
		
		var nForm = nContentWindow.document.createElement( 'form' );
		nForm.setAttribute( 'method', 'post' );
		
		/* Add POST data */
		var aoPost = this.aoRequests[ this.iRequestingIndex ].aoPost;
		for ( var i=0 ; i<aoPost.length ; i++ )
		{
			nInput = nContentWindow.document.createElement( 'input' );
			nInput.setAttribute( 'name', aoPost[i].sName );
			nInput.setAttribute( 'type', 'text' );
			nInput.value = aoPost[i].sValue;
			
			nForm.appendChild( nInput );
		}
		
		/* Add GET data tot he URL */
		var aoGet = this.aoRequests[ this.iRequestingIndex ].aoGet;
		var sUrlAddition = '';
		for ( var i=0 ; i<aoGet.length ; i++ )
		{
			sUrlAddition += aoGet[i].sName+'='+aoGet[i].sValue+'&';
		}
		
		nForm.setAttribute( 'action', 
			this.aoRequests[ this.iRequestingIndex ].sUrl +'?'+ sUrlAddition );
		
		
		/* Add the form and the iframe */
		nContentWindow.document.body.appendChild( nForm );
		
		/* Send the request */
		nForm.submit();
		
		/* For now we presume XSS */
		this._fnReply();
		
		return 0;
	},
	
	
	
	/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
	 * API functions
	 */
	
	/*
	 * Function: fnRequest
	 * Purpose:  Add a request to the queue to be run
	 * Returns:  -
	 * Inputs:   int:iIndex - request index to send
	 */
	fnRequest: function ( iIndex )
	{
		this.aQueue[this.aQueue.length++] = iIndex;
		this._fnQueue();
	},
	
	
	/*
	 * Function: fnNewRequest
	 * Purpose:  Create a new Remoting request
	 * Returns:  int:iIndex - Index of the request in the queue
	 * Inputs:   string:sUrl - Target URL
	 *           function:fnCallback - Callback function
	 *           objecgt:oObj - Object to pass on callback
	 *           bool:bScope - Make 'oObj' the scape of the callback
	 */
	fnNewRequest: function ( sUrl, fnCallback, oObj, bScope )
	{
		this.aoRequests[this.aoRequests.length++] = {
			'sUrl': sUrl,
			'aoPost': new Array(),
			'aoGet': new Array(),
			'fnCallback': fnCallback,
			'oObj': oObj,
			'bScope': bScope };
		
		return (this.aoRequests.length-1);
	},
	
	
	/*
	 * Function: fnAddGet
	 * Purpose:  Add a 'get' variable to a request
	 * Returns:  -
	 * Inputs:   int:iIndex - request index to add the variable to
	 *           string:sName - name of the variable
	 *           string:sValue - variable value
	 */
	fnAddGet: function ( iIndex, sName, sValue )
	{
		if ( this.aoRequests[iIndex] == 'undefined' )
		{
			return -1;
		}
		
		this.aoRequests[iIndex].aoGet[ this.aoRequests[iIndex].aoGet.length++ ] = {
			'sName': sName,
			'sValue': encodeURIComponent( sValue ) };
		return 0;
	},
	
	
	/*
	 * Function: fnAddPost
	 * Purpose:  Add a 'post' variable to a request
	 * Returns:  -
	 * Inputs:   int:iIndex - request index to add the variable to
	 *           string:sName - name of the variable
	 *           string:sValue - variable value
	 */
	fnAddPost: function ( iIndex, sName, sValue )
	{
		if ( this.aoRequests[iIndex] == 'undefined' )
		{
			return -1;
		}
		
		this.aoRequests[iIndex].aoPost[ this.aoRequests[iIndex].aoPost.length++ ] = {
			'sName': sName,
			'sValue': sValue };
		return 0;
	}
}